@@ -1,7 +1,8 @@
 """Gives users direct access to class and functions."""
 from logical.logical import\
     logical,\
-    unary, binary, every,\
+    nullary, unary, binary, every,\
+    nf_, nt_,\
     uf_, id_, not_, ut_,\
     bf_,\
     and_, nimp_, fst_, nif_, snd_, xor_, or_,\
