@@ -1,14 +1,17 @@
 """
-Callable subclass of the tuple type for representing logical operators and
-connectives based on their truth tables.
+Callable subclass of the built-in :obj:`tuple <tuple>` type for representing
+logical operators and connectives based on their truth tables.
 
-All four unary and all sixteen binary operators are available as attributes of
-the :obj:`logical` class, and also as constants. Likewise, the three sets of
-operators :obj:`logical.unary`, :obj:`logical.binary`, and :obj:`logical.every`
-are available both as attributes of :obj:`logical` and as constants.
+The two nullary, four unary, and sixteen binary operators are available as
+attributes of the :obj:`logical` class, and also as constants. Likewise, the
+four sets of operators :obj:`logical.nullary`, :obj:`logical.unary`,
+:obj:`logical.binary`, and :obj:`logical.every` are available both as attributes
+of :obj:`logical` and as exported top-level constants.
 """
 from __future__ import annotations
+from typing import Sequence, Union
 import doctest
+from collections.abc import Iterable
 import itertools
 import math
 
@@ -46,8 +49,11 @@ class logical(tuple):
     >>> f(1, 0)
     1
 
-    Pre-defined instances are defined for all unary and binary functions, and
-    are available as attributes of this class and as top-level constants:
+    Pre-defined instances are defined for all nullary, unary and binary functions,
+    and are available as attributes of this class and as top-level constants:
+
+    * ``(0,)`` = :obj:`logical.nf_` represents **NULLARY FALSE**
+    * ``(1,)`` = :obj:`logical.nt_` represents **NULLARY TRUE**
 
     * ``(0, 0)`` = :obj:`logical.uf_` represents **UNARY FALSE**
     * ``(0, 1)`` = :obj:`logical.id_` represents **IDENTITY**
@@ -76,8 +82,8 @@ class logical(tuple):
     >>> and_(1, 0)
     0
 
-    Because this class is derived from the ``tuple`` type, all methods
-    and functions that operate on tuples also work with instances of
+    Because this class is derived from the :obj:`tuple <tuple>` type, all
+    methods and functions that operate on tuples also work with instances of
     this class.
 
     >>> logical((1, 0)) == logical((1, 0))
@@ -87,7 +93,9 @@ class logical(tuple):
     >>> logical((1, 0))[1]
     0
     """
-    names = {
+    names: set = {
+        (0,): 'nf',
+        (1,): 'nt',
         (0, 0): 'uf',
         (0, 1): 'id',
         (1, 0): 'not',
@@ -109,21 +117,30 @@ class logical(tuple):
         (1, 1, 1, 0): 'nand',
         (1, 1, 1, 1): 'bt'
     }
-    """Typical concise names for all unary and binary operators."""
+    """Typical concise names for all nullary, unary, and binary operators."""
 
-    every = None
-    """Set of all unary and binary operators."""
+    nullary: set = {} # Populated at top-level, after this class definition.
+    """Set of all nullary operators."""
 
-    unary = None
+    unary: set = {} # Populated at top-level, after this class definition.
     """Set of all unary operators."""
 
-    binary = None
+    binary: set = {} # Populated at top-level, after this class definition.
     """Set of all binary operators."""
 
-    def __call__(self: logical, *arguments) -> int:
-        """
-        Apply this operator to an input tuple.
+    every: set = {} # Populated at top-level, after this class definition.
+    """Set of all nullary, unary, and binary operators."""
 
+    def __call__(self: logical, *arguments: Union[Sequence[int], Sequence[Sequence[int]]]) -> int:
+        """
+        Apply the function represented by this instance to zero or more integer
+        arguments (where the arguments collectively represent an individual input
+        row within a truth table) or to a single iterable of integers (where the
+        entries of the iterable represent an individual input row within a truth
+        table).
+
+        >>> logical((1,))()
+        1
         >>> logical((1, 0))(1)
         0
         >>> logical((1, 0, 0, 1))(0, 0)
@@ -136,21 +153,61 @@ class logical(tuple):
         0
         >>> logical((1, 0, 0, 1, 0, 1, 0, 1))(1, 1, 0)
         0
+        >>> logical((1, 0, 0, 1, 0, 1, 0, 1))([1, 1, 0])
+        0
+        >>> logical((1, 0, 0, 1, 0, 1, 0, 1))((1, 1, 0))
+        0
+        >>> logical((1, 0, 0, 1, 0, 1, 0, 1))((1, 1, 0))
+        0
+
+        Any attempt to apply an instance to an invalid input raises an exception.
+
+        >>> logical((1, 0))(2.3)
+        Traceback (most recent call last):
+          ...
+        TypeError: expecting zero or more integers or a single iterable of integers
+        >>> logical((1, 0))(['abc'])
+        Traceback (most recent call last):
+          ...
+        TypeError: expecting zero or more integers or a single iterable of integers
+        >>> logical((1, 0))(2)
+        Traceback (most recent call last):
+          ...
+        ValueError: expecting an integer that is 0 or 1
         """
+        if len(arguments) == 1 and isinstance(arguments[0], Iterable):
+            arguments = arguments[0]
+
+        if not all(isinstance(argument, int) for argument in arguments):
+            raise TypeError(
+                'expecting zero or more integers or a single iterable of integers'
+            )
+
+        if not all(argument in (0, 1) for argument in arguments):
+            raise ValueError(
+                'expecting an integer that is 0 or 1'
+            )
+
+        if len(arguments) == 0:
+            return self[0]
         if len(arguments) == 1: # pylint: disable=R1705
             return self[[0, 1].index(arguments[0])]
-        elif len(arguments) == 2:
+        if len(arguments) == 2:
             return self[[(0, 0), (0, 1), (1, 0), (1, 1)].index(tuple(arguments))]
-        else:
-            inputs = list(itertools.product(*[(0, 1)]*self.arity()))
-            return self[inputs.index(tuple(arguments))]
+
+        inputs = list(itertools.product(*[(0, 1)]*self.arity()))
+        return self[inputs.index(tuple(arguments))]
 
     def name(self: logical) -> str:
         """
         Return the typical concise name for this operator.
 
+        >>> logical((0,)).name()
+        'nf'
         >>> logical((1, 0, 0, 1)).name()
         'xnor'
+        >>> len([o.name for o in logical.nullary])
+        2
         >>> len([o.name for o in logical.unary])
         4
         >>> len([o.name for o in logical.binary])
@@ -162,6 +219,8 @@ class logical(tuple):
         """
         Return the arity of this operator.
 
+        >>> logical((1,)).arity()
+        0
         >>> logical((1, 0)).arity()
         1
         >>> logical((1, 0, 0, 1)).arity()
@@ -169,7 +228,29 @@ class logical(tuple):
         """
         return int(math.log2(len(self)))
 
-    uf_ = None
+    nf_: logical = None
+    """
+    Nullary **FALSE** (constant) operation.
+
+    +-----------+
+    | ``nf_()`` |
+    +-----------+
+    | ``0``     |
+    +-----------+
+    """
+
+    nt_: logical = None
+    """
+    Nullary **TRUE** (constant) operation.
+
+    +-----------+
+    | ``nt_()`` |
+    +-----------+
+    | ``1``     |
+    +-----------+
+    """
+
+    uf_: logical = None
     """
     Unary **FALSE** (constant) operation.
 
@@ -182,7 +263,7 @@ class logical(tuple):
     +-------+------------+
     """
 
-    id_ = None
+    id_: logical = None
     """
     Unary **IDENTITY** operation.
 
@@ -195,7 +276,7 @@ class logical(tuple):
     +-------+------------+
     """
 
-    not_ = None
+    not_: logical = None
     """
     Unary **NOT** operation (*i.e.*, negation).
 
@@ -208,7 +289,7 @@ class logical(tuple):
     +-------+-------------+
     """
 
-    ut_ = None
+    ut_: logical = None
     """
     Unary **TRUE** (constant) operation.
 
@@ -221,7 +302,7 @@ class logical(tuple):
     +-------+------------+
     """
 
-    bf_ = None
+    bf_: logical = None
     """
     Binary **FALSE** (constant) operation.
 
@@ -238,7 +319,7 @@ class logical(tuple):
     +------------+---------------+
     """
 
-    and_ = None
+    and_: logical = None
     """
     Binary **AND** operation (*i.e.*, conjunction).
 
@@ -255,7 +336,7 @@ class logical(tuple):
     +------------+----------------+
     """
 
-    nimp_ = None
+    nimp_: logical = None
     """
     Binary **NIMP** operation (*i.e.*, ``>``).
 
@@ -272,7 +353,7 @@ class logical(tuple):
     +------------+-----------------+
     """
 
-    fst_ = None
+    fst_: logical = None
     """
     Binary **FST** operation (*i.e.*, first/left-hand input).
 
@@ -289,7 +370,7 @@ class logical(tuple):
     +------------+----------------+
     """
 
-    nif_ = None
+    nif_: logical = None
     """
     Binary **NIF** operation (*i.e.*, ``<``).
 
@@ -306,7 +387,7 @@ class logical(tuple):
     +------------+----------------+
     """
 
-    snd_ = None
+    snd_: logical = None
     """
     Binary **SND** operation (*i.e.*, second/right-hand input).
 
@@ -323,7 +404,7 @@ class logical(tuple):
     +------------+----------------+
     """
 
-    xor_ = None
+    xor_: logical = None
     """
     Binary **XOR** operation (*i.e.*, ``!=``).
 
@@ -340,7 +421,7 @@ class logical(tuple):
     +------------+----------------+
     """
 
-    or_ = None
+    or_: logical = None
     """
     Binary **OR** operation (*i.e.*, disjunction).
 
@@ -357,7 +438,7 @@ class logical(tuple):
     +------------+---------------+
     """
 
-    nor_ = None
+    nor_: logical = None
     """
     Binary **NOR** operation.
 
@@ -374,7 +455,7 @@ class logical(tuple):
     +------------+----------------+
     """
 
-    xnor_ = None
+    xnor_: logical = None
     """
     Binary **XNOR** operation (*i.e.*, ``==``).
 
@@ -391,7 +472,7 @@ class logical(tuple):
     +------------+-----------------+
     """
 
-    nsnd_ = None
+    nsnd_: logical = None
     """
     Binary **NSND** operation (*i.e.*, negation of second/right-hand input).
 
@@ -408,7 +489,7 @@ class logical(tuple):
     +------------+-----------------+
     """
 
-    if_ = None
+    if_: logical = None
     """
     Binary **IF** operation.
 
@@ -425,7 +506,7 @@ class logical(tuple):
     +------------+---------------+
     """
 
-    nfst_ = None
+    nfst_: logical = None
     """
     Binary **NFST** operation (*i.e.*, negation of first/left-hand input).
 
@@ -442,7 +523,7 @@ class logical(tuple):
     +------------+-----------------+
     """
 
-    imp_ = None
+    imp_: logical = None
     """
     Binary **IMP** operation (*i.e.*, implication or ``<=``).
 
@@ -459,7 +540,7 @@ class logical(tuple):
     +------------+----------------+
     """
 
-    nand_ = None
+    nand_: logical = None
     """
     Binary **NAND** operation (*i.e.*, negation of conjunction).
 
@@ -476,7 +557,7 @@ class logical(tuple):
     +------------+-----------------+
     """
 
-    bt_ = None
+    bt_: logical = None
     """
     Binary **TRUE** (constant) operation.
 
@@ -493,7 +574,9 @@ class logical(tuple):
     +------------+---------------+
     """
 
-# All unary and binary operators as named class constants.
+# All nullary, unary, and binary operators as named class constants.
+logical.nf_ = logical((0,))
+logical.nt_ = logical((1,))
 logical.uf_ = logical((0, 0))
 logical.id_ = logical((0, 1))
 logical.not_ = logical((1, 0))
@@ -515,29 +598,32 @@ logical.imp_ = logical((1, 1, 0, 1))
 logical.nand_ = logical((1, 1, 1, 0))
 logical.bt_ = logical((1, 1, 1, 1))
 
-# All unary and binary operators as top-level constants.
-uf_ = logical.uf_
-id_ = logical.id_
-not_ = logical.not_
-ut_ = logical.ut_
-bf_ = logical.bf_
-and_ = logical.and_
-nimp_ = logical.nimp_
-fst_ = logical.fst_
-nif_ = logical.nif_
-snd_ = logical.snd_
-xor_ = logical.xor_
-or_ = logical.or_
-nor_ = logical.nor_
-xnor_ = logical.xnor_
-nsnd_ = logical.nsnd_
-if_ = logical.if_
-nfst_ = logical.nfst_
-imp_ = logical.imp_
-nand_ = logical.nand_
-bt_ = logical.bt_
+# All nullary, unary, and binary operators as top-level constants.
+nf_: logical = logical.nf_
+nt_: logical = logical.nt_
+uf_: logical = logical.uf_
+id_: logical = logical.id_
+not_: logical = logical.not_
+ut_: logical = logical.ut_
+bf_: logical = logical.bf_
+and_: logical = logical.and_
+nimp_: logical = logical.nimp_
+fst_: logical = logical.fst_
+nif_: logical = logical.nif_
+snd_: logical = logical.snd_
+xor_: logical = logical.xor_
+or_: logical = logical.or_
+nor_: logical = logical.nor_
+xnor_: logical = logical.xnor_
+nsnd_: logical = logical.nsnd_
+if_: logical = logical.if_
+nfst_: logical = logical.nfst_
+imp_: logical = logical.imp_
+nand_: logical = logical.nand_
+bt_: logical = logical.bt_
 
-# Useful containers.
+# Useful class constants: containers of all operators.
+logical.nullary = {nf_, nt_}
 logical.unary = {uf_, id_, not_, ut_}
 logical.binary = {
     bf_,
@@ -545,11 +631,13 @@ logical.binary = {
     nor_, xnor_, nsnd_, if_, nfst_, imp_, nand_,
     bt_
 }
-logical.every = logical.unary | logical.binary
+logical.every = logical.nullary | logical.unary | logical.binary
 
-unary = logical.unary
-binary = logical.binary
-every = logical.every
+# Top-level constants corresponding to class constants for containers.
+nullary: set = logical.nullary
+unary: set = logical.unary
+binary: set = logical.binary
+every: set = logical.every
 
 if __name__ == "__main__":
     doctest.testmod() # pragma: no cover
